@@ -1,6 +1,6 @@
 package Spreadsheet::ParseXLSX;
 our $AUTHORITY = 'cpan:DOY';
-$Spreadsheet::ParseXLSX::VERSION = '0.18';
+$Spreadsheet::ParseXLSX::VERSION = '0.19';
 use strict;
 use warnings;
 use 5.008;
@@ -12,10 +12,18 @@ use Scalar::Util 'openhandle';
 use Spreadsheet::ParseExcel 0.61;
 use XML::Twig;
 
+use Spreadsheet::ParseXLSX::Decryptor;
+
 
 
 sub new {
-    bless {}, shift;
+    my $class = shift;
+    my (%args) = @_;
+
+    my $self = bless {}, $class;
+    $self->{Password} = $args{Password} if defined $args{Password};
+
+    return $self;
 }
 
 
@@ -25,11 +33,23 @@ sub parse {
 
     my $zip = Archive::Zip->new;
     my $workbook = Spreadsheet::ParseExcel::Workbook->new;
+
+    if ($self->_check_signature($file)) {
+        $file = Spreadsheet::ParseXLSX::Decryptor->open(
+            $file,
+            $self->{Password}
+        );
+    }
+
     if (openhandle($file)) {
         bless $file, 'IO::File' if ref($file) eq 'GLOB'; # sigh
-        $zip->readFromFileHandle($file) == Archive::Zip::AZ_OK
+        my $fh = ref($file) eq 'File::Temp'
+            ? IO::File->new("<&=" . fileno($file))
+            : $file;
+        $zip->readFromFileHandle($fh) == Archive::Zip::AZ_OK
             or die "Can't open filehandle as a zip file";
         $workbook->{File} = undef;
+        $workbook->{__tempfile} = $file;
     }
     elsif (!ref($file)) {
         $zip->read($file) == Archive::Zip::AZ_OK
@@ -43,6 +63,25 @@ sub parse {
     return $self->_parse_workbook($zip, $workbook, $formatter);
 }
 
+sub _check_signature {
+    my $self = shift;
+    my ($file) = @_;
+
+    my $signature = '';
+    if (openhandle($file)) {
+        bless $file, 'IO::File' if ref($file) eq 'GLOB'; # sigh
+        $file->read($signature, 2);
+        $file->seek(-2, IO::File::SEEK_CUR);
+    }
+    elsif (!ref($file)) {
+        my $fh = IO::File->new($file, 'r');
+        $fh->read($signature, 2);
+        $fh->close;
+    }
+
+    return $signature eq "\xd0\xcf";
+}
+
 sub _parse_workbook {
     my $self = shift;
     my ($zip, $workbook, $formatter) = @_;
@@ -92,6 +131,7 @@ sub _parse_workbook {
               _Book    => $workbook,
               _SheetNo => $idx,
           );
+          $sheet->{SheetHidden} = 1 if defined $_->att('state') and $_->att('state') eq 'hidden';
           $self->_parse_sheet($sheet, $files->{sheets}{$idx});
           ($sheet)
         } else {
@@ -123,7 +163,9 @@ sub _parse_sheet {
 
     my @column_formats;
     my @column_widths;
+    my @columns_hidden;
     my @row_heights;
+    my @rows_hidden;
 
     my $default_row_height   = 15;
     my $default_column_width = 10;
@@ -228,6 +270,7 @@ sub _parse_sheet {
                 for my $colnum ($col->att('min')..$col->att('max')) {
                     $column_widths[$colnum - 1] = $col->att('width');
                     $column_formats[$colnum - 1] = $col->att('style');
+                    $columns_hidden[$colnum - 1] = $col->att('hidden');
                 }
 
                 $twig->purge;
@@ -237,6 +280,7 @@ sub _parse_sheet {
                 my ( $twig, $row ) = @_;
 
                 $row_heights[ $row->att('r') - 1 ] = $row->att('ht');
+                $rows_hidden[ $row->att('r') - 1 ] = $row->att('hidden');
 
                 $twig->purge;
             },
@@ -371,10 +415,12 @@ sub _parse_sheet {
     $sheet->{RowHeight} = [
         map { defined $_ ? 0+$_ : 0+$default_row_height } @row_heights
     ];
+    $sheet->{RowHidden} = \@rows_hidden;
     $sheet->{ColWidth} = [
         map { defined $_ ? 0+$_ : 0+$default_column_width } @column_widths
     ];
     $sheet->{ColFmtNo} = \@column_formats;
+    $sheet->{ColHidden} = \@columns_hidden;
 
 }
 
@@ -690,6 +736,7 @@ sub _extract_files {
     my $wb_name = ($rels->find_nodes(
         qq<//Relationship[\@Type="$type_base/officeDocument"]>
     ))[0]->att('Target');
+    $wb_name =~ s{^/}{};
     my $wb_xml = $self->_parse_xml($zip, $wb_name);
 
     my $path_base = $self->_base_path_for($wb_name);
@@ -698,25 +745,33 @@ sub _extract_files {
         $self->_rels_for($wb_name)
     );
 
+    my $get_path = sub {
+        my ($p) = @_;
+
+        return $p =~ s{^/}{}
+            ? $p
+            : $path_base . $p;
+    };
+
     my ($strings_xml) = map {
-        $zip->memberNamed($path_base . $_->att('Target'))->contents
+        $zip->memberNamed($get_path->($_->att('Target')))->contents
     } $wb_rels->find_nodes(qq<//Relationship[\@Type="$type_base/sharedStrings"]>);
 
     my $styles_xml = $self->_parse_xml(
         $zip,
-        $path_base . ($wb_rels->find_nodes(
+        $get_path->(($wb_rels->find_nodes(
             qq<//Relationship[\@Type="$type_base/styles"]>
-        ))[0]->att('Target')
+        ))[0]->att('Target'))
     );
 
     my %worksheet_xml = map {
-        if ( my $sheetfile = $zip->memberNamed($path_base . $_->att('Target'))->contents ) {
+        if ( my $sheetfile = $zip->memberNamed($get_path->($_->att('Target')))->contents ) {
             ( $_->att('Id') => $sheetfile );
         }
     } $wb_rels->find_nodes(qq<//Relationship[\@Type="$type_base/worksheet"]>);
 
     my %themes_xml = map {
-        $_->att('Id') => $self->_parse_xml($zip, $path_base . $_->att('Target'))
+        $_->att('Id') => $self->_parse_xml($zip, $get_path->($_->att('Target')))
     } $wb_rels->find_nodes(qq<//Relationship[\@Type="$type_base/theme"]>);
 
     return {
@@ -872,7 +927,7 @@ Spreadsheet::ParseXLSX - parse XLSX files
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =head1 SYNOPSIS
 
@@ -888,9 +943,17 @@ This module is an adaptor for L<Spreadsheet::ParseExcel> that reads XLSX files.
 
 =head1 METHODS
 
-=head2 new
+=head2 new(%opts)
 
-Returns a new parser instance. Takes no parameters.
+Returns a new parser instance. Takes a hash of parameters:
+
+=over 4
+
+=item Password
+
+Password to use for decrypting encrypted files.
+
+=back
 
 =head2 parse($file, $formatter)
 
